import numpy as np
from smbus2 import SMBus, i2c_msg
from time import sleep
import ov5642 as ov

OV5642_I2C_ADDR = 0x3C
UB934_I2C_ADDR = 0x30
UB933_I2C_ADDR = 0x58
SER_I2C_ADDR = UB933_I2C_ADDR + 1

def cam_i2c_write(reg_addr, data):
    command = [reg_addr >> 8, reg_addr & 0xff, data]
    write = i2c_msg.write(OV5642_I2C_ADDR, command)
    with SMBus(0) as i2c:
        i2c.i2c_rdwr(write)
        # print('Register write ov5642 address=0x%04x data=0x%02x' %(reg_addr, data))

def cam_i2c_read(reg_addr):
    command = [reg_addr >> 8, reg_addr & 0xff]
    write = i2c_msg.write(OV5642_I2C_ADDR, command)
    read = i2c_msg.read(OV5642_I2C_ADDR, 1)
    with SMBus(0) as i2c:
        i2c.i2c_rdwr(write, read)
        data = list(read)
        # print('Register read ov5642 address=0x%04x data=0x%02x' %(reg_addr, data[0]))

def ser_i2c_read(reg_addr):
    with SMBus(0) as i2c:
        data = i2c.read_byte_data(SER_I2C_ADDR, reg_addr)
        # print('Register read ds90ub933 address=0x%02x data=0x%02x' %(reg_addr, data))
    return data

def ser_i2c_write(reg_addr, data):
    with SMBus(0) as i2c:
        i2c.write_byte_data(SER_I2C_ADDR, reg_addr, data)
        # print('Register write ds90ub933 address=0x%02x data=0x%02x' %(reg_addr, data))

def des_i2c_read(reg_addr):
    with SMBus(0) as i2c:
        data = i2c.read_byte_data(UB934_I2C_ADDR, reg_addr)
        # print('Register read ds90ub934 address=0x%02x data=0x%02x' %(reg_addr, data))
    return data

def des_i2c_write(reg_addr, data):
    with SMBus(0) as i2c:
        i2c.write_byte_data(UB934_I2C_ADDR, reg_addr, data)
        # print('Register write ds90ub934 address=0x%02x data=0x%02x' %(reg_addr, data))

def serdes_init():
    des_i2c_read(0x00)  # I2C device ID
    des_i2c_read(0x04)  # device status
    des_i2c_write(0x01, 0x07)  # device reset
    sleep(0.001)
    des_i2c_read(0x04)  # device status

    des_i2c_write(0x4c, 0x01)  # port0 write enable
    rdata = des_i2c_read(0x5b)
    print('ds90ub934 remote serializer ID=0x%02x' %rdata);

    des_i2c_write(0x58, 0x58)  # Pass-thorough enabled
    des_i2c_write(0x5c, SER_I2C_ADDR<<1)  # ser alias ID 0
    des_i2c_write(0x5d, OV5642_I2C_ADDR<<1)  # slave ID 0
    # des_i2c_write(0x65, OV5642_I2C_ADDR<<1|0x01);  # slave alias ID 0, auto ack
    des_i2c_write(0x65, OV5642_I2C_ADDR<<1)  # slave alias ID 0

    # des_i2c_read(0x6d)  # PORT_CONFIG
    des_i2c_write(0x6d, 0x02)  # mode override, RAW12HF

    des_i2c_read(0x4d)

    ser_i2c_read(0x00)  # I2C device ID
    ser_i2c_write(0x01, 0x33)  # device reset
    sleep(0.001)

    ser_i2c_read(0x0c)  # status
    ser_i2c_write(0x05, 0x22)  # mode override, 12-bit mode
    ser_i2c_read(0x05)  # mode select
    ser_i2c_write(0x11, 0x64)  # SCL High Time=5us
    ser_i2c_write(0x12, 0x64)  # SCL Low Time=5us
    ser_i2c_read(0x0c)  # status

def ov5642_init():
    for name in ov.ov5642:
        cam_i2c_write(name['adrs'], name['data'])

def cam_init():
    print('Camera init start')

    serdes_init()

    des_i2c_write(0x65, OV5642_I2C_ADDR<<1|0x01);  # slave alias ID 0, auto ack
    cam_i2c_write(0x3017, 0x6f)  # PAD OUTPUT ENABLE00, pclk disable
    des_i2c_write(0x65, OV5642_I2C_ADDR<<1)  # slave alias ID 0

    ov5642_init()

    des_i2c_write(0x65, OV5642_I2C_ADDR<<1|0x01);  # slave alias ID 0, auto ack
    cam_i2c_write(0x3017, 0x7f)  # PAD OUTPUT ENABLE00, pclk enable
    des_i2c_write(0x65, OV5642_I2C_ADDR<<1)  # slave alias ID 0

    rdata = ser_i2c_read(0x0c)  # status

    print('Ser status:0x%x' %rdata)
    print('Camera init finished')
